package com.usky.iot.controller.web;


import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.log.annotation.Log;
import com.usky.common.log.enums.BusinessType;
import com.usky.iot.domain.PmWorkContent;
import com.usky.iot.domain.PmWorkReport;
import com.usky.iot.service.PmWorkContentService;
import com.usky.iot.service.vo.WorkTimeExportVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDate;
import java.time.temporal.TemporalAdjusters;
import java.util.List;

/**
 * <p>
 * 工作内容表 前端控制器
 * </p>
 *
 * @author fu
 * @since 2024-05-20
 */
@RestController
@RequestMapping("/pmWorkContent")
public class PmWorkContentController {
    @Autowired
    private PmWorkContentService pmWorkContentService;

    /**
     * 项目名
     *
     * @param startDate
     * @param endDate
     * @param projectName
     * @return
     */
    @GetMapping("/projectName")
    public ApiResult<List<PmWorkContent>> pageList(@RequestParam(value = "startDate", required = false) String startDate,
                                                   @RequestParam(value = "endDate", required = false) String endDate,
                                                   @RequestParam(value = "projectName", required = false) String projectName,
                                                   @RequestParam(value = "projectAscription", required = false, defaultValue = "1") Integer projectAscription) {
        return ApiResult.success(pmWorkContentService.projectQuery(startDate, endDate, projectName, projectAscription));
    }

    /**
     * 报告记录
     *
     * @param startDate
     * @param endDate
     * @param projectName
     * @param reportId
     * @param projectAscription
     * @return
     */
    @GetMapping("/reportRecord")
    public ApiResult<List<PmWorkReport>> reportRecord(@RequestParam(value = "startDate", required = false) String startDate,
                                                      @RequestParam(value = "endDate", required = false) String endDate,
                                                      @RequestParam(value = "projectName", required = false) String projectName,
                                                      @RequestParam(value = "reportId", required = false) Integer reportId,
                                                      @RequestParam(value = "projectAscription", required = false, defaultValue = "1") Integer projectAscription) {
        if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
            startDate = LocalDate.now().toString();
            endDate = LocalDate.now().toString();
        }
        return ApiResult.success(pmWorkContentService.workReportQuery(startDate, endDate, projectName, projectAscription, reportId));
    }

    /**
     * 报告详情分页列表
     *
     * @param projectAscription 1:我负责；2：我收到；3：我发出
     * @param pageNum           页数
     * @param pageSize          页大小
     * @return
     */
    @GetMapping("/page")
    public ApiResult<CommonPage<PmWorkReport>> page(@RequestParam(value = "projectAscription", required = false, defaultValue = "2") Integer projectAscription,
                                                    @RequestParam(value = "pageNum") Integer pageNum,
                                                    @RequestParam(value = "pageSize") Integer pageSize,
                                                    @RequestParam(value = "reportId", required = false) Integer reportId,
                                                    @RequestParam(value = "startDate", required = false) String startDate,
                                                    @RequestParam(value = "endDate", required = false) String endDate,
                                                    @RequestParam(value = "projectId", required = false) Integer projectId) {
        return ApiResult.success(pmWorkContentService.reportPage(projectAscription, pageNum, pageSize, reportId, startDate, endDate, projectId));
    }

    /**
     * 工时统计
     *
     * @param userId
     * @param projectId
     * @param filter
     * @param startDate
     * @param endDate
     * @param workerOrProject
     * @return
     */
    @GetMapping("/workHourStatistic")
    public ApiResult<List<Object>> workHourStatistic(@RequestParam(value = "userId", required = false) Long userId,
                                                     @RequestParam(value = "projectId", required = false) Integer projectId,
                                                     @RequestParam(value = "filter", required = false, defaultValue = "1") Integer filter,
                                                     @RequestParam(value = "startDate", required = false) String startDate,
                                                     @RequestParam(value = "endDate", required = false) String endDate,
                                                     @RequestParam(value = "workerOrProject", required = false, defaultValue = "1") Integer workerOrProject) {
        if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)) {
            startDate = LocalDate.now().with(TemporalAdjusters.firstDayOfYear()).toString();
            endDate = LocalDate.now().with(TemporalAdjusters.lastDayOfYear()).toString();
        }
        return ApiResult.success(pmWorkContentService.workHourStatistic(userId, projectId, filter, startDate, endDate, workerOrProject));
    }

    /**
     * 工时明细导出
     *
     * @param response
     * @param userId
     * @param projectId
     * @param startDate
     * @param endDate
     * @throws IOException
     */
    @Log(title = "工时统计", businessType = BusinessType.EXPORT)
    @PostMapping("/workHourStatisticExport")
    public void export(HttpServletResponse response,
                       @RequestParam(value = "userId", required = false) Long userId,
                       @RequestParam(value = "projectId", required = false) Integer projectId,
                       @RequestParam(value = "startDate", required = false) String startDate,
                       @RequestParam(value = "endDate", required = false) String endDate) throws IOException {
        List<WorkTimeExportVO> list = pmWorkContentService.workHourStatisticExport(userId, projectId, startDate, endDate);
        ExcelUtil<WorkTimeExportVO> util = new ExcelUtil<WorkTimeExportVO>(WorkTimeExportVO.class);
        util.exportExcel(response, list, "工时统计", "工时统计");
    }

}

